;--------------------------------------------------------------------;
; Program create_shapefile_tiff_20070301.pro
;
; CREATE_SHAPE_FILE
;
;
; This program is designed to read in a binary, gridded, floating point
;  file.  It searches a directory for files named *.yyyymmdd - where
;  yyyymmdd represents yesterdays date - and then writes the contents
;  to a shapefile and GeoTiff.  You must modify the command line arguments
;  in the final line in the code (that calls the procedure) to match your
;  input file parameters.
;
; Syntax:
;   CREATE_SHAPE_FILE,minlat=integer,maxlat=integer,minlon=integer,
;                     maxlon=integer,cellsize=decimal,xsize=integer,
;                     ysize=integer,filename='string',inputdir='string',
;                     outputdir='string',usersphere=decimal,
;                     endiantype='big or little_endian',
;                     processdate='string',/verbose
;
; Keywords:
;    See the command line information at the end of the script.
;
; Written by:  Viviane Silva, Denis Nadeau, Tim Love
;
; Modified:  2006 12 19 VS
;            2007 01 11 DN
;            2007 01 15 TL for documentation
;            2007 01 19 TL for modifications based on 20070117 meeting, including
;                       addition of command line arguments, byteorder switching,
;                       spherical datum, and manual processing date
;            2007 02 18 DN for file projection adaptation
;            2007 02 28 TL for correct GeoTiff keys and Spheroids.
;
;-------------------------------------------------------------------------------

PRO create_shape_file, $
    MINLAT = minlat, $
    MINLON = minlon, $
    MAXLAT = maxlat, $
    MAXLON = maxlon, $
    CELLSIZE = cellsize, $
    XSIZE    = XSize, $
    YSIZE    = YSize, $
    FILENAME = filename, $
    INPUTDIR = inputdir, $
    OUTPUTDIR = outputdir, $
    USERSPHERE = usersphere, $
    ENDIANTYPE = endiantype, $
    PROCESSDATE = processdate, $
    VERBOSE = verbose, $
    PIXMAP  = pixmap

    DEVICE, RETAIN=2, DECOMPOSED=0


    IF N_ELEMENTS(inputDir) GT 0 THEN BEGIN
        IF FILE_TEST(inputDir[0], /DIRECTORY) THEN $
            inDir = inputDir[0] $
        ELSE $
            CD, CURRENT = inDir
    ENDIF ELSE $
        CD, CURRENT = inDir
    IF N_ELEMENTS(outputDir) GT 0 THEN BEGIN
        IF FILE_TEST(outputDir[0], /DIRECTORY) THEN $
            outDir = outputDir[0] $
        ELSE $
            CD, CURRENT = outDir
    ENDIF ELSE $
        CD, CURRENT = outDir

    verbose = KEYWORD_SET(verbose)


    ;*********** Find yesterday's date. *************************************************
    CALDAT, SYSTIME(/JULIAN)-1, month, day, year
    fileStr = STRTRIM(year,2) + $
        (month GE 10 ? STRTRIM(month,2):'0' + STRTRIM(month,2)) + $
        (day GE 10 ? STRTRIM(day,2):'0' + STRTRIM(day,2))

    ;*********** For setting the date manually via command line argument ******
    IF processDate NE 0 THEN $
      fileStr = processDate


    ;*********** Find all input files in the directory *********************************
    inputFiles = FILE_SEARCH(inDir + PATH_SEP() + fileName  , COUNT = nFiles)

    IF nFiles EQ 0 THEN BEGIN
        MESSAGE, /INFO, 'No data files found in directory "' + inDir + '"'
        RETURN
    ENDIF

    ;******** Define the output file names. *********************************************
    outFileshp = outDir + fileName + '.shp'
    outFileprj = outDir + fileName + '.prj'
    outFiletif = outDir + fileName + '.tif'

    ;****** Define the contour intervals ************************************************
    ;******  Change these as needed. ****************************************************
    endVals = [0.1,1.0,2.0,5.0,10.0,15.0,20.0,$
             30.0,40.0,50.0,75.0]


    ;******* Read in the binary data. ***************************************************
    inputDims = [XSize,YSize]
    OPENR, lun, inputFiles, /GET_LUN
    inputData = FLTARR(inputDims[0],inputDims[1])
    READU, lun, inputData
    FREE_LUN, lun
    CATCH, /CANCEL
    ;********* Swap the byteorder if necessary. ******************************************
    IF endianType EQ "big_endian" THEN $
       inputData = SWAP_ENDIAN(TEMPORARY(inputData))

    ;********* Convert to millimeters (uncomment if necessary.) **************************
    ;;;inputData = temporary(inputData)

    ;*********** These pixel resolution values are now parameters of Create_shape ********
    DeltaLat=cellsize
    DeltaLon=cellsize

    ;************ Compute the latitude and longitude for each pixel **********************
    lon=findgen(XSize)*DeltaLon+(maxlon)
    lat=findgen(YSize)*DeltaLat+(minlat)

    ;************ Compute contour paths **************************************************
    Data=congrid(bytscl(inputData,2,100),XSize,YSize)


    ;********** Compute contour paths for SHAPEFILE CREATION *****************************
    CONTOUR, inputData, lon,lat,LEVELS = endVals,path_info=info,$
             path_xy=xy,/overplot,/downhill,/follow,/path_data_coord


    ;********** Display data to screen ***************************************************
    IF NOT KEYWORD_SET(pixmap) THEN BEGIN

        ;*********** Create a display window and set the width/height in pixels **********
        window,xsize=XSize,ysize=YSize

        ;*********** Set Geographic information for this window, including the coordinate
        ;***********  conversion and map projection.  All expressed in lat/lon. **********
        map_set,/cylindrical,limit=[minlat,maxlon,maxlat,minlon], $
                /noborder,xmargin=0,ymargin=0,color=150

        loadct,39

        tv,Data

        ;********** Display the Grid ****************************************************
        map_grid,color=150,/label

        ;********** Display the Continents **********************************************
        map_continents,color=150

        ;********** Retrieve the color palette ******************************************
        tvlct, r, g, b, /get

        for i = 0, (n_elements(info) - 1 ) do begin
            s = [indgen(info(i).n), 0]
            ;******* Plot the contours only:
    ;        Plots, xy(*,info(i).offset + s ), /data
            ;;;print, info(i).offset
        endfor
    ENDIF
    ;*********** End display data to screen. ********************************************



    ;************************************************************************************
    ;*********** If you want to see contour information use verbose: ********************
    IF KEYWORD_SET(verbose) THEN BEGIN
        help, xy
        help, info, /struct
        print, ' '
        print, 'Number of polygons in this file:', n_elements(info)
        print, info.value
    ENDIF
    ;************************************************************************************



    ;************************************************************************************
    ;********* Write the GeoTIFF File ***************************************************

       ;******** Create tag and geokey information **************************************
       g_tags = { $
               ModelPixelScaleTag: [DeltaLat, DeltaLon, 0D], $ ;(I,J,X)
               ModelTiepointTag: [0.0, 0.0, 0.0, maxlon, maxlat, 0.0D], $ ;(I,J,K,X,Y,Z)
               GTModelTypeGeoKey: 2, $ ; (2=latitude-longitude System)
               GTRasterTypeGeoKey: 1, $ ; (1= RasterPixelIsArea )
               GeogCitationGeoKey:   "Geographic Perfect Sphere", $
               GeogAngularUnitsGeoKey: 9102S, $ ; (Degrees)
               GeographicTypeGeoKey: 4035, $ ; Perfect Sphere
               GeogGeodeticDatumGeoKey: 6035 $ ; Perfect Sphere
             }

       ;******** The tiff needs to rotate date from 270 degrees and transpose for GEOTIF *
       ;******** Data are saved as floating point
       TiffData=rotate(inputData,7)

       ;******** Write out the tiff file. ***********************************************
       write_tiff,outFiletif,TiffData,/float,geotif=g_tags

    ;********* End Geotiff creation. ****************************************************
    ;************************************************************************************



    ;************************************************************************************
    ;********* Write the Shapefile ******************************************************

    ;********* Create the new shapefile and define the entity type to Polygon ***********
    ;*********  entity = 1 (point), = 3 (polyline), = 5 (polygon)
    oShape=OBJ_NEW('IDLffShape', outFileshp, /UPDATE, ENTITY_TYPE=5)

    ;******** Set the attribute definition for the new shapefile ************************
    ;******** Format is: IDLffShape::AddAttribute,
    ;          'NameOfYourAttribute',
    ;          TypeOfValue (3:Integer, 7:String text),
    ;          NumberOfCharactersForValue, (The # of spaces of your largest value)
    ;          PRECISION = 0
    ;******** You may need to alter the NumberOfCharactersForValue to match your data. **
    oShape->IDLffShape::AddAttribute, 'Contour', 3, 10, $
      PRECISION=0
    oShape->IDLffShape::AddAttribute, 'nVerts', 3, 10, $
        PRECISION=0
    oShape->IDLffShape::AddAttribute,'Title',7,34,$
        PRECISION = 0
    oShape->IDLffShape::AddAttribute,'Unit',7,10,$
      PRECISION = 0
    oShape->IDLffShape::AddAttribute,'Resolution',7,10,$
      PRECISION = 0

    ;******* Make an entity for every contour (part) that was produced ******************
    for i=0, (n_elements(info)-1) do begin

        ;********** Create a structure for new entity ***********************************
        entNew = {IDL_SHAPE_ENTITY}

        ;********* Define the values for the new entity *********************************
        entnew.SHAPE_TYPE=5
        indices_lower = info[i].offset
        indices_upper = (info[i].offset + info[i].n-1)
        nElements = indices_upper - indices_lower
        nVerts = make_array(2,nElements+2,/double)
        nVerts[*,0:nElements] = xy[*, indices_lower : indices_upper]
        last = n_elements(nVerts) / 2 - 1
        nVerts[*,last] = nVerts[*,0]
        entNew.N_Vertices = info[i].n+1
        entNew.Vertices = ptr_new(nVerts)

        entNew.BOUNDS[0] = minlon ;min x
        entNew.BOUNDS[1] = minlat ;min y
        entNew.BOUNDS[2] = 0.00000000 ;min z
        entNew.BOUNDS[3] = 0.00000000 ;min measure
        entNew.BOUNDS[4] = maxlon ;max x
        entNew.BOUNDS[5] = maxlat ;max y
        entNew.BOUNDS[6] = 0.00000000 ;max z
        entNew.BOUNDS[7] = 0.00000000 ;max measure

        ;********** Create a structure for new attributes *******************************
        attrNew = oShape ->IDLffShape::GetAttributes(/ATTRIBUTE_STRUCTURE)

        ;********* Define the values for the new attributes *****************************
        ;********* These correspond to the attribute parameters you set up earlier. You *
        ;*********  will need to alter attributes 2,3,4 below to match your data. *******
        attrNew.ATTRIBUTE_0 = info[i].value ;contour values
        attrNew.ATTRIBUTE_1 = info[i].n ;number of vertex pair
        attrnew.ATTRIBUTE_2='NOAA/CPC RFE2 Africa Region'
        attrnew.ATTRIBUTE_3='mm'
        attrnew.ATTRIBUTE_4='0.1x0.1 degree'

        ;******** Add the new entity to the new shapefile *******************************
        oShape->IDLffShape::PutEntity, entNew

        ;******** Add the attributes to the new shapefile *******************************
        oShape ->SetAttributes, i, attrNew

    endfor
    ;********* End the contour-entity creation loop *************************************
    ;********* Close the shapefile ******************************************************
    OBJ_DESTROY, oShape


    ;********* Set the data projection.  Geographic Spheriod is used here. **************
    OPENW, PRJ, Outfileprj, /GET_LUN
    PRINTF, PRJ, 'GEOGCS["GCS_WGS_1984_Perfect_Sphere",DATUM["D_Sphere",SPHEROID["Sphere",6371000.0,0.0]],PRIMEM["Greenwich",0.0],UNIT["Degree",0.0174532925199433]]'
    FREE_LUN, PRJ


;************ End the program! **********************************************************
END



;********** Setup the program calls for various applications: ***************************

;minlat:      Minimum latitude of the dataset
;maxlat:      Maximum latitude of the dataset
;minlon:      Minimum longitude of the dataset
;maxlon:      Maximum longitude of the dataset
;cellcise:    The resolution in degrees of the dataset
;xsize:       Number of pixels in the x-axis direction
;ysize:       Number of pixels in the y-axis direction
;filename:    The partial name of the dataset.  The characters occurring before the YYYYMMDD date stamp.
;inputdir:    The location of the input files
;outputdir:   The location for the output files
;usersphere:  The radius of the Earth, as used in the input data.  For default, enter 6371000.0  For GFS model data, use 6371200.0
;endiantype:  The type of machine endian.  'little_endian' for Intel, Linux, Sgi.  'big_endian' for HP, SUN
;processdate: Enter '0' if you are using the default date (yesterday), or 'YYYYMMDD' for your choice of date.



    ;********** For cron job
   ;; CREATE_SHAPE_FILE,minlat=-90,maxlat=90,minlon=-180,maxlon=180,cellsize=1,xsize=360,ysize=180,filename='sst_io.bin.',inputdir='/cpc/home/giswg/sst_iov2/',outputdir='/cpc/home/giswg/sst_iov2/',usersphere='6371000.0',endiantype='big_endian',processdate='0',/verbose,/pixmap

    ;********** Without a cron job
   ; CREATE_SHAPE_FILE,minlat=-90,maxlat=90,minlon=-180,maxlon=180,cellsize=1,xsize=360,ysize=180,filename='sst_io.bin.',inputdir='/cpc/home/giswg/sst_iov2/',outputdir='/cpc/home/giswg/sst_iov2/',usersphere='6371000.0',endiantype='big_endian',processdate='0',/verbose

    ;********** Without a cron job and without displaying to the screen
    ;;CREATE_SHAPE_FILE,minlat=-90,maxlat=90,minlon=-180,maxlon=180,cellsize=1,xsize=360,ysize=180,filename='sst_io.bin.',inputdir='/cpc/home/giswg/sst_iov2/',outputdir='/cpc/home/giswg/sst_iov2/',usersphere='6371000.0',endiantype='big_endian',processdate='0',/verbose,/pixmap

    ;********** Without contour information do not use verbose
    ;;;CREATE_SHAPE_FILE,minlat=-40,maxlat=40,minlon=55,maxlon=-20,cellsize=0.1,xsize=751,ysize=801,filename='all_products.bin.',inputdir='/export/fews/wd52tl/IDL_SCRIPTS/RFE_DATA/',outputdir='/export/fews/wd52tl/IDL_SCRIPTS/RFE_DATA/',usersphere='6371000.0',endiantype='big_endian',processdate='0'



;END
;*****************************************************************************************
;*****************************************************************************************

